package internal

import (
	"encoding/json"
	"gitea.cdlsxd.cn/sdk/plugin/proto"
	"gitee.com/chengdu_blue_brothers/openapi-go-sdk-v2/api"
	"gitee.com/chengdu_blue_brothers/openapi-go-sdk-v2/notify"
	"gitee.com/chengdu_blue_brothers/openapi-go-sdk-v2/sdk"
	"plugins/utils/helper"
	"plugins/zltxv2/internal/vo"
)

type Config struct {
	AppId     string `json:"app_id"`
	AppKey    string `json:"app_key"`
	BaseUri   string `json:"base_uri"`
	NotifyUrl string `json:"notify_url"`
}

type Card struct {
	Number   string `json:"number"`
	Password string `json:"password"`
}

func (c *Card) ToJson() []byte {
	bytes, _ := json.Marshal(c)
	return bytes
}

func transConfig(config []byte) (*Config, error) {
	var c Config
	err := json.Unmarshal(config, &c)
	if err != nil {
		return nil, err
	}
	return &c, nil
}

func (c *Config) client() (*sdk.Client, error) {
	scheme := getScheme(c.BaseUri)
	domain, err := getDomain(c.BaseUri, scheme)
	if err != nil {
		return nil, err
	}
	return sdk.NewClientWithAccessKey(c.AppKey, scheme, domain)
}

func (c *Config) orderReq(in *proto.OrderRequest) *api.OrderCreateReq {
	accountType := int64(0)
	if in.Order.Account == "" {
		accountType = 0
	} else if helper.IsPhoneNumber(in.Order.Account) {
		accountType = 1
	} else if helper.IsEmail(in.Order.Account) {
		accountType = 2
	}
	return &api.OrderCreateReq{
		OutTradeNo:      in.Order.OrderNo,       // 商户侧订单号长度只能是1-64位
		ProductId:       in.Product.ProductNo,   // 产品编码不能为空
		Number:          int(in.Order.Quantity), // 部分商品存在限购数量，具体可联系商务
		NotifyUrl:       c.NotifyUrl,            // 订单回调通知地址长度不能为空，且长度不能超过255
		Extends:         string(in.Order.Extra), // 部分特殊商品需要，具体可联系商务
		RechargeAccount: in.Order.Account,       // 账号类型：直充类型商品必传,商品为卡密类型时，则表示接收卡密的手机号
		AccountType:     accountType,            // 直充类型商品必传 - 1：手机号 - 2:  QQ 号
		Attach:          "",                     // 透传参数，当传了此参数时，查询和回调接口中将原样携带此参数
		UnitPrice:       0,                      // 交易商品单价，单位：元（最多保留4位小数） - 0或不传表示不限制 - 当此价格小于采购价时，下单失败
	}
}

func orderResp(request *proto.OrderRequest, resp *api.OrderCreateResp) *proto.OrderResponse {
	data, _ := json.Marshal(resp)
	return &proto.OrderResponse{
		Result: &proto.Result{
			Status:  proto.Status_ING,
			OrderNo: request.Order.OrderNo,
			TradeNo: resp.OrderNo, // 平台订单号
			Message: "成功",
			Data:    data,
		},
	}
}

func (c *Config) queryReq(in *proto.QueryRequest) *api.OrderQueryReq {
	return &api.OrderQueryReq{
		OutTradeNo: in.Order.OrderNo,
	}
}

func queryResp(appKey string, request *proto.QueryRequest, resp *api.OrderQueryResp) (*proto.QueryResponse, error) {
	data, err := json.Marshal(resp)
	if err != nil {
		return nil, err
	}
	status := vo.OrderStatus(resp.TradeStatus)
	pb := &proto.QueryResponse{
		Result: &proto.Result{
			Status:  status.GetOrderStatus(),
			OrderNo: request.Order.OrderNo,
			TradeNo: resp.OrderNo,
			Message: resp.TradeStateDesc,
			Data:    data,
			Extra:   nil,
		},
	}
	if len(resp.Cards) > 0 {
		c, err := getQueryCard(appKey, &resp.Cards[0])
		if err != nil {
			return nil, err
		}
		pb.Result.Extra = c.ToJson()
	}
	return pb, nil
}

func getQueryCard(appKey string, card *api.Cards) (*Card, error) {
	// - 1：卡密+卡密：no和pwd都不为空
	// - 2：卡密，pwd不为空
	// - 3：链接，url不为空
	car := &Card{}
	if card.CardType == 1 {
		pwd, err := decryptAES(card.Pwd, appKey)
		if err != nil {
			return nil, err
		}
		no, err := decryptAES(card.No, appKey)
		if err != nil {
			return nil, err
		}
		car.Password = pwd
		car.Number = no
	} else if card.CardType == 2 {
		pwd, err := decryptAES(card.Pwd, appKey)
		if err != nil {
			return nil, err
		}
		car.Password = pwd
	} else if card.CardType == 3 {
		// 无
	}
	return car, nil
}

func notifyResp(appKey string, resp *notify.OrderReq) (*proto.NotifyResponse, error) {
	data, err := json.Marshal(resp)
	if err != nil {
		return nil, err
	}
	status := vo.OrderStatus(resp.TradeStatus)
	pb := &proto.NotifyResponse{
		Result: &proto.Result{
			Status:  status.GetOrderStatus(),
			OrderNo: resp.OutTradeNo,
			TradeNo: resp.OrderNo,
			Message: resp.TradeStateDesc,
			Data:    data,
			Extra:   nil,
		},
	}
	b, err := getNotifyCard(appKey, resp.Cards)
	if err != nil {
		return nil, err
	}
	pb.Result.Extra = b
	return pb, nil
}

func getNotifyCard(appKey string, cards []notify.Cards) ([]byte, error) {
	if len(cards) == 0 {
		return nil, nil
	}
	card := cards[0]
	car := &Card{}
	if card.CardType == 1 {
		pwd, err := decryptAES(card.Pwd, appKey)
		if err != nil {
			return nil, err
		}
		no, err := decryptAES(card.No, appKey)
		if err != nil {
			return nil, err
		}
		car.Password = pwd
		car.Number = no
	} else if card.CardType == 2 {
		pwd, err := decryptAES(card.Pwd, appKey)
		if err != nil {
			return nil, err
		}
		car.Password = pwd
	} else if card.CardType == 3 {
		// 无
	}
	return car.ToJson(), nil
}
